// JavaScript Document
var plotlyLoaded = false;
var plotlyLoading = false;
var plotInstance;
var xDomainMin = -10;
var xDomainMax = 10;
var yDomainMin;
var yDomainMax;
var plotlyGD;

var plotColors = ["#000000", "#0f5066", "#a6077e", "#e81531", "#b89a16", "#1a3796", "#e85015", "#15e8cc", "#16b849", "#66240f", "#fc8505", "#cc08bf", "#25660f", "#1a868a", "#ff2200", "#5eff00", "#5e141e", "#808080", "#FF0000", "#800000", "#FFFF00", "#808000", "#00FF00", "#008000", "#00FFFF", "#008080", "#0000FF", "#000080", "#FF00FF", "#800080", "#C0C0C0"];
//Plot Functions
var plotFunctions = [{ fname: 'plotFunction', params: 1 },
					{ fname: 'plotParametric', params: 4 },
					{ fname: 'plotPolar', params: 3 },
					{ fname: 'setX', params: 2 },
					{ fname: 'setY', params: 2 }
];

var instance;

function computeYScale(xScale) {
    if (yDomainMin != null && yDomainMax != null) {
        return [yDomainMin, yDomainMax];
    }
    else {
        var width = $("#plotPopupContent").width();
        var height = $("#plotPopupContent").height();
        var xDiff = xScale[1] - xScale[0];
        var yDiff = height * xDiff / width;
        return [-yDiff / 2, yDiff / 2];
    }
}

function drawCartesian(data, xDomainMin, xDomainMax) {
    try {
        var plotInput = {
            target: '#plot',
            data: data,
            grid: true,
            xAxis: {
                label: 'x',
                domain: [xDomainMin, xDomainMax]
            },
            yAxis: {
                label: 'y',
                domain: computeYScale([xDomainMin, xDomainMax]),
            },
            width: document.getElementById("plotContainer").clientWidth * 0.95,
            height: document.getElementById("plotContainer").clientHeight * 0.95
        };

        instance = functionPlot(plotInput);

        return instance;
    }
    catch (err) {
        console.log(err);
        alert(err);
    }
}

function getFunction(graphType, eq) {
    return [{
        fn: eq,
        scope: config.scope,
        title: "y=" + eq,
        sampler: 'builtIn',
        fnType: 'linear',
        graphType: graphType
    }];
}

function getParametric(graphType, x, y, rangeMin, rangeMax) {
    return [{
        x: x,
        y: y,
        scope: config.scope,
        title: "x=" + x + " , y=" + y,
        sampler: 'builtIn',
        range: [rangeMin, rangeMax],
        fnType: 'parametric',
        graphType: graphType
    }];
}

function getPolar(graphType, r, rangeMin, rangeMax) {
    return [{
        r: r,
        scope: config.scope,
        title: "r=" + r,
        sampler: 'builtIn',
        range: [rangeMin, rangeMax],
        fnType: 'polar',
        graphType: graphType
    }];
}

function clearPlot() {
    plotData = [];
    plotDiv.innerHTML = "";
    addTextToHistory(getText("plot_cleared"));

    showCaret();
}

function listPlot() {
    var i = 0;
    var toDisplay = "";
    for (i = 0; i < plotData.length; i++) {
        toDisplay = toDisplay + (i + 1) + ") " + plotData[i].title + "<br>";
    }
    showCaret();
    return toDisplay;
}

function drawCartesianPlot() {
    //set color
    var i = 0;
    for (i = 0; i < plotData.length; i++) {
        plotData[i].color = plotColors[i];
    }

    return drawCartesian(plotData, xDomainMin, xDomainMax);
}

function setX(xMin, xMax) {
    xDomainMin = math.eval(xMin);
    xDomainMax = math.eval(xMax);
    return "x range=(" + xMin + "," + xMax + ")";
}

function setY(yMin, yMax) {
    yDomainMin = math.eval(yMin);
    yDomainMax = math.eval(yMax);
    return "y range=(" + yMin + "," + yMax + ")";
}

function plotParametric(ft, gt, tRangeMin, tRangeMax) {
    if (plotData.length < plotColors.length) {
        plotData = plotData.concat(getParametric('polyline', ft, gt, math.eval(tRangeMin), math.eval(tRangeMax)));
        return listPlot();
    }
    else {
        return "Error: " + plotColors.length + " fn max";
    }
}

function plotPolar(r, rRangeMin, rRangeMax) {
    if (plotData.length < plotColors.length) {
        plotData = plotData.concat(getPolar('polyline', r, math.eval(rRangeMin), math.eval(rRangeMax)));
        return listPlot();
    }
    else {
        return "Error: " + plotColors.length + " fn max";
    }
}

function plotFunction(fx) {
    if (plotData.length < plotColors.length) {
        plotData = plotData.concat(getFunction('polyline', fx));

        return listPlot();
    }
    else {
        return "Error: " + plotColors.length + " fn max";
    }
}

function plot() {
    plotDiv.innerHTML = "";

    window.setTimeout(function () { modal.style.display = "none"; }, 250);
    var mySpan = document.getElementById('plotClose');


    plotPopup.style.display = "block";

    plotInstance = drawCartesianPlot();

    $(mySpan).hammer().off('tap press');
    $(mySpan).hammer().bind('tap press', function () {
        caretEnabled = true;
        ADSIntersitialShow();
        window.setTimeout(function () { plotPopup.style.display = "none"; }, 250);
    });

    var shareButton = document.getElementById('plotShare');
    $(shareButton).hammer().off('tap press');
    $(shareButton).hammer().bind('tap press', function () {
        shareImage(document.getElementById("plot"), listPlot());
    });

    return listPlot();
}

/*
function mesh(x, y, z) {
   if (plotlyLoaded) {
    window.setTimeout(function () { modal.style.display = "none"; }, 250);
    var mySpan = document.getElementById('plotClose3D');

    plotPopup3D.style.display = "block";

    // Plotting the mesh
    var data = [
        {
            type: 'mesh3d',
            x: x,
            y: y,
            z: z,
            intensity: [0, 0.33, 0.66, 1],
            colorscale: [
            [0, 'rgb(255, 0, 255)'],
            [0.5, 'rgb(0, 255, 0)'],
            [1, 'rgb(0, 0, 255)']
            ]
        }
    ];
    var width = $("#plotPopupContent3D").width();
    var height = $("#plotPopupContent3D").height();

    var layout = {
        width: width,
        height: height,
        margin: {
            l: 0,
            r: 0,
            b: 0,
            t: 0,
        }
    };

    Plotly.newPlot("plot3D", data, layout, { displayModeBar: false, });

    $(mySpan).hammer().bind('tap press', function () {
        caretEnabled = true;
        window.setTimeout(function () { plotPopup3D.style.display = "none"; }, 250);
    });
  }
    else
    {

        loadPlotly(function () { mesh(x, y, z);});

    }
    return "ok";
}
*/

function loadPlotly(cback)
{
    if (!plotlyLoaded && !plotlyLoading)
    {
        console.log(window.location.pathname);
        plotlyLoading = true;
        $.getScript("scripts/plotly-latest.min.js", function () { plotlyLoaded = true; cback() });


    }

}

function surf(f, x, y) {
    
    if (plotlyLoaded) {
        var maxPoints = 5000000;
        plotDiv3D.innerHTML = "";
        x = math.flatten(x._data);
        y = math.flatten(y._data);

        var dotsToDraw = x.length * y.length;
        console.log(dotsToDraw);
        if (dotsToDraw > maxPoints) {
            var er = {};
            er.message = "Too many points to draw, max = " + maxPoints; throw er;
        }

        var z = [];
        var i = 0;
        var j = 0;

        for (i = 0; i < y.length; i++) {
            var newRow = [];
            for (j = 0; j < x.length; j++) {
                newRow.push(f(x[j], y[i]));
            }

            z.push(newRow);
        }

        // console.log(z);

        window.setTimeout(function () { modal.style.display = "none"; }, 250);
        var mySpan = document.getElementById('plotClose3D');



        plotPopup3D.style.display = "block";

        var data = [
            {
                type: 'surface',
                z: z,
                x: x,
                y: y,
            }
        ];
        var width = $("#plotPopupContent3D").width();
        var height = $("#plotPopupContent3D").height();

        var layout = {
            width: width,
            height: height,
            margin: {
                l: 0,
                r: 0,
                b: 0,
                t: 0,
            }
        };

        try {
            Plotly.newPlot("plot3D", data, layout, { displayModeBar: false, }).then(function (gd) {
                return Plotly.toImage(gd);
            }).then(function (dataURI) {
                var shareButton = document.getElementById('plotShare3D');
                $(shareButton).hammer().off('tap press');
                $(shareButton).hammer().bind('tap press', function () {
                    shareImageURL(dataURI, "Math Plus Ultra");
                });
            });
        } catch (e)
        {
            alert("3D surf is not supported in this device");
        }

 
        $(mySpan).hammer().off('tap press');
        $(mySpan).hammer().bind('tap press', function () {
            caretEnabled = true;
            window.setTimeout(function () { plotPopup3D.style.display = "none"; }, 250);
        });

        
    }
    else
    {

        loadPlotly(function () { surf(f, x, y);});

    }

    return "ok";
}

function plotXY(x, y) {
    if (plotlyLoaded) {
    plotDiv3D.innerHTML = "";
    x = math.flatten(x._data);
    y = math.flatten(y._data);

    window.setTimeout(function () { modal.style.display = "none"; }, 250);
    var mySpan = document.getElementById('plotClose3D');

    var shareButton = document.getElementById('plotShare3D');
    $(shareButton).hammer().off('tap press');
    $(shareButton).hammer().bind('tap press', function () {
        shareImage(document.getElementById("plot3D"), "Math Plus Ultra");
    });

    plotPopup3D.style.display = "block";


    var trace1 = {
        x: x,
        y: y,
        mode: 'lines+markers',
        type: 'scatter'
    };

    var data = [trace1];

    var width = $("#plotPopupContent3D").width();
    var height = $("#plotPopupContent3D").height();

    var layout = {
        width: width,
        height: height,
        margin: {
            l: 60,
            r: 0,
            b: 30,
            t: 0,
        }
    };

   
    Plotly.newPlot("plot3D", data, layout, { displayModeBar: false, });

    $(mySpan).hammer().bind('tap press', function () {
        caretEnabled = true;
        window.setTimeout(function () { plotPopup3D.style.display = "none"; }, 250);
    });
    }
    else {

        loadPlotly(function () { plotXY(x, y); });

    }

    return "ok";
}

/*
function contour(f, x, y) {
if (plotlyLoaded) {
    var maxPoints = 100000;
    plotDiv3D.innerHTML = "";
    x = math.flatten(x._data);
    y = math.flatten(y._data);

    var dotsToDraw = x.length * y.length;
    console.log(dotsToDraw);
    if (dotsToDraw > maxPoints) {
        var er = {};
        er.message = "Too many points to draw, max = " + maxPoints; throw er;
    }

    var z = [];
    var i = 0;
    var j = 0;

    for (i = 0; i < y.length; i++) {
        var newRow = [];
        for (j = 0; j < x.length; j++) {
            newRow.push(f(x[j], y[i]));
        }

        z.push(newRow);
    }

    console.log(z);

    window.setTimeout(function () { modal.style.display = "none"; }, 250);
    var mySpan = document.getElementById('plotClose3D');

        var shareButton = document.getElementById('plotShare3D');
        $(shareButton).hammer().off('tap press');
        $(shareButton).hammer().bind('tap press', function () {
            shareImage(document.getElementById("plot3D"), "Math Plus Ultra");
        });

    plotPopup3D.style.display = "block";

    var data = [
        {
            type: 'contour',
            z: z,
            x: x,
            y: y,
        }
    ];
    var width = $("#plotPopupContent3D").width();
    var height = $("#plotPopupContent3D").height();

    var layout = {
        width: width,
        height: height,
        margin: {
            l: 40,
            r: 0,
            b: 20,
            t: 10,
        }
    };

    Plotly.newPlot("plot3D", data, layout, { displayModeBar: false, });

    $(mySpan).hammer().bind('tap press', function () {
        caretEnabled = true;
        window.setTimeout(function () { plotPopup3D.style.display = "none"; }, 250);
    });
 }
    else {

        loadPlotly(function () { contour(f, x, y); });

    }

    return "ok";
}
*/

math.import({
    plot: plot,
    plotFunction: plotFunction,
    plotParametric: plotParametric,
    plotPolar: plotPolar,
    setX: setX,
    setY: setY,
    //mesh: mesh,
    surf: surf,
    //contour: contour,
    plotXY: plotXY
});